#!/usr/bin/python
# -*- coding: utf-8 -*-

# attention: pour les accents, ce fichier doit être impérativement
#            lu et modifié sous éditeur avec encodage UTF-8

################################################################
######                                                    ######
######          Module additionnel: bibgene.py            ######
######                                                    ######
######            accompagnant la CALCULEXT               ######
######      [Calculatrice Extensible Version 1.40]        ######
######                                                    ######
################################################################

# Copyright (C) 2008 Jean-Paul Vidal dit "Tyrtamos" — Tous droits réservés.
#   Logiciel distribué sur le site <http://linux.jpvweb.com>
#
#   Ce programme est un logiciel libre. Vous pouvez le redistribuer ou le
#   modifier suivant les termes de la "GNU General Public License" version 3,
#   telle que publiée par la Free Software Foundation.
#
#   Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
#   AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
#   ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
#   Publique GNU pour plus de détails.
#
#   Ce programme doit être distribué avec une copie de la
#   License Générale Publique GNU (fichier license.html).
#   Si ce n’est pas le cas, consultez: <http://www.gnu.org/licenses/gpl.html>.

##################################################################
# Attention: configuration volontairement inhabituelle pour python
# les divisions entre 2 entiers seront décimales et pas entières!
# par exemple: 10/3 donnera "3.33333333333333" et pas "3"
# (les divisions entières seront désormais faites avec "//")
#
from __future__ import division

##################################################################
# Importation des modules système
#

##################################################################
# Importation des modules nécessaires aux calculs du présent module
#
from math import *
from copy import *
from decimal import *

from sys import modules
pmain = modules['__main__']

###############################################################################
# Renvoie une liste triée selon l'ordre alphanumérique
#
def triliste(L,direct=True):
    """triliste(L,direct=True): renvoie la liste triée dans le sens alphanumérique direct (direct=True) ou inverse (direct=False)"""
    L.sort()
    if not direct:
        L.reverse()
    return L

###############################################################################
# Construit une liste de n paramètres ou de n listes de paramètres de L1 (début) à L2 (fin)
#  si L1 et L2 sont des nombres, donne le résultat sous forme de liste de n valeurs
#  si L1 et L2 sont des listes, donne le résultat sous forme de liste de n listes de valeurs
#
def jeuxdessais(n,L1,L2):
    """jeudessais(n,L1,L2): Construit une liste de n listes de paramètres de L1 (début) à L2 (fin)"""
    # vérification de la validité de n
    if (not pmain.estentier(n)) or (n<2):
        raise ValueError ("erreur jeudessais(n,L1,L2): n doit être un nombre entier >= 2")
    L=[]
    if not pmain.estliste(L1):
        # si L1 n'est pas une liste, c'est forcément un nombre
        if (not pmain.estnombre(L1)):
            raise ValueError ("erreur jeudessais(n,L1,L2): L1 doit être soit un nombre soit une liste")
        # ici, L1 est un nombre, alors L2 doit en être un aussi
        if (not pmain.estnombre(L2)):
            raise ValueError ("erreur jeuxdessais(n,L1,L2), comme L1 est un nombre, L2 doit en être un aussi")
        # calcul d'une liste simple (et non d'une liste de liste)
        L2m1=L2-L1
        for i in range(0,n):
            L.append(L1+L2m1/(n-1)*i)
    else:
        # L1 est une liste, L2 doit en être une aussi!
        if not pmain.estliste(L2):
            raise ValueError ("erreur jeuxdessais(n,L1,L2), comme L1 est une liste, L2 doit en être une aussi")
        if L1==[]:
            raise ValueError ("erreur jeuxdessais(n,L1,L2), L1 doit être une liste non vide")
        np=len(L1)
        if np!=len(L2):
            raise ValueError ("erreur jeuxdessais(n,L1,L2), L1 et L2 n'ont pas le même nombre de membres")
        # calcul d'une liste de n listes de np paramètres compris entre L1 et L2
        for i in range(0,n):
            Lp=[]
            for k in range(0,np):
                Lp.append(L1[k]+(L2[k]-L1[k])/(n-1)*i)
            L.append(Lp)
    return L

###############################################################################
# Calcule le volume restant de liquide d'une cuve cylindique horizontale
#  en fonction de son diamètre d, de son volume total vt et du niveau du liquide h
#  exemple d'application: cuve de mazout
#
def volcuve(h,d,vt):
    """volcuve(h,d,vt): Calcule le volume restant de liquide d'une cuve cylindique horizontale de diamètre d et de volume total vt"""
    # vérification des données
    if (not pmain.estnombre(h)) or (h<0):
        raise ValueError ("erreur volcuve(h,d,vt): h doit être un nombre positif ou nul")
    if (not pmain.estnombre(d)) or (d<=0):
        raise ValueError ("erreur volcuve(h,d,vt): d doit être un nombre positif")
    if (not pmain.estnombre(vt)) or (vt<=0):
        raise ValueError ("erreur volcuve(h,d,vt): vt doit être un nombre positif")
    if h>d:
        raise ValueError ("erreur volcuve(h,d,vt): h ne peut pas être supérieur à d (=ça déborde!)")
    # calcul
    if h==0:
        return 0
    r=d/2
    if h==r:
        return vt/2
    if h==d:
        return vt
    if h>r:
        h=d-h
        nivhaut=True
    else:
        nivhaut=False
    st=pi*r*r
    alpha=acos((r-h)/r)
    sr=(r-h)*sqrt(r*r-(r-h)**2)
    sa=st/pi*alpha
    sh=sa-sr
    v=vt*sh/st
    if nivhaut:
        v=vt-v
    return v

###############################################################################
# Résolution d'une équation implicite par recherche du zéro d'une fonction
#
def fnimplicite(fn,x1=0,dx=1,eps=1.0e-15):
    """fnimplicite(fn,x1,dx,eps=1.0e-15): résolution d'une équation implicite par recherche du zéro d'une fonction"""
    # vérification des données
    if not pmain.estfonction(fn):
        raise ValueError ("erreur fnimplicite(fn,x1,dx,eps=1.0e-15): fn doit être un nom de fonction")
    if not pmain.estnombre(x1):
        raise ValueError ("erreur fnimplicite(fn,x1,dx,eps=1.0e-15): x1 doit être un nombre")
    if not pmain.estnombre(dx):
        raise ValueError ("erreur fnimplicite(fn,x1,dx,eps=1.0e-15): dx doit être un nombre")
    if not pmain.estnombre(eps):
        raise ValueError ("erreur fnimplicite(fn,x1,dx,eps=1.0e-15): eps doit être un nombre")
    # calcul de la 1ère valeur de la fonction pour le x initial
    f1=fn(x1)
    i=0  # initialisation du compteur de boucle
    while True:
        # calcul d'une 2ème valeur pour le x précédent + l'incrément
        x2=x1+dx
        f2=fn(x2)
        if abs(f2)<eps:
            # on a atteint la précision demandée: renvoyer x2
            break
        if ((f1>0)and(f2>0))or((f1<0)and(f2<0)):
            #f1 et f2 ont le même signe, ils sont donc du même côté de l'abscisse
            if abs(f2)>abs(f1):
                # on s'éloigne du zéro: revenir en arrière avec un incrément diminué
                dx*=-0.5
        else:
            # on a dépassé le zéro: revenir en arrière avec un incrément diminué
            dx*=-0.5
        # on vérifie qu'on n'est pas en train de boucler pour rien
        i+=1
        if i>1000:
            raise ValueError ("erreur fnimplicite(fn,x1,dx,eps=1.0e-15): impossible de trouver le zero de cette fonction")
        # on fait en sorte que le nouveau point soit désormais le point de base
        x1=x2
        f1=f2
        # et on boucle pour un nouvel essai!
    return x2

###############################################################################
# Résolution d'une équation implicite par recherche du zéro d'une fonction sans changement de sens
#
def fnimplicite2(fn,x1=0,dx=1,eps=1.0e-15):
    """fnimplicite(fn,x1,dx,eps=1.0e-15): résolution d'une équation implicite par recherche du zéro d'une fonction sans changement de sens"""
    # vérification des données
    if not pmain.estfonction(fn):
        raise ValueError ("erreur fnimplicite2(fn,x1,dx,eps=1.0e-15): fn doit être un nom de fonction")
    if not pmain.estnombre(x1):
        raise ValueError ("erreur fnimplicite(fn,x1,dx,eps=1.0e-15): x1 doit être un nombre")
    if not pmain.estnombre(dx):
        raise ValueError ("erreur fnimplicite(fn,x1,dx,eps=1.0e-15): dx doit être un nombre")
    if not pmain.estnombre(eps):
        raise ValueError ("erreur fnimplicite(fn,x1,dx,eps=1.0e-15): eps doit être un nombre")
    # calcul de la 1ère valeur de la fonction pour le x initial
    f1=fn(x1)
    i=0  # initialisation du compteur de boucle
    while True:
        i+=1
        if i>1000:
            raise ValueError ("erreur fnimplicite(fn,x1,dx,eps=1.0e-15): impossible de trouver le zero de cette fonction")
        # calcul d'une 2ème valeur pour le x précédent + l'incrément
        x2=x1+dx
        f2=fn(x2)
        if abs(f2)<eps:
            # on a atteint la précision demandée: renvoyer x2
            break
        if ((f1>0)and(f2>0))or((f1<0)and(f2<0)):
            x1=x2
            f1=f2
        else:
            #f1 et f2 sont de part et d'autre de l'abscisse
            dx*=0.5
    return x2
